;
;*******************************************************
;*******************************************************
;**                                                   **
;**    Sztuka programowania mikrokontrolerow AVR      **
;**                                                   **
;**         Program obslugi klawiatury                **
;**                  wariant 1                        **
;**                                                   **
;**       Andrzej Pawluczuk                           **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                Bialystok, 2006                    **
;**                                                   **
;*******************************************************
;*******************************************************
;
;*******************************************************
	.nolist
.include "m8515def.inc"
	.list
	.listmac
;-----------------------------------------------------------------------------
.macro	pushf
	push	acc
	in	acc,sreg
	push	acc
.endm
;-----------------------------------------------------------------------------
.macro	popf
	pop	acc
	out	sreg,acc
	pop	acc
.endm
;-----------------------------------------------------------------------------
.macro	pushz
	push	zl
	push	zh
.endm
;-----------------------------------------------------------------------------
.macro	popz
	pop	zh
	pop	zl
.endm
;-----------------------------------------------------------------------------
.macro	ldz
	ldi	zl,low(@0)
	ldi	zh,high(@0)
.endm
;-----------------------------------------------------------------------------
.macro	ldx
	ldi	xl,low(@0)
	ldi	xh,high(@0)
.endm
;
;*******************************************************
;*******************************************************
;**                                                   **
;**   Uzywane w programie rejestry                    **
;**   Znaczenie poszczegolnych bitow                  **
;**                                                   **
;*******************************************************
;*******************************************************
;
.equ	Stand_TIMSK	=	0b00000010
;	Timer/counter		  ||| | ||
;	Interrupt Mask		  ||| | |+-- OCIE0 : Timer/Counter0 Output Compare Match Interrupt Enable
;				  ||| | +--- TOIE0 : Timer/Counter 0 Overflow Interrupt Enable
;				  ||| +----- TICIE1 : Timer/Counter1 Input Capture Interrupt Enable
;				  ||+------- OCIE1B : Timer/counter1 Output Compare B Match Interrupt Enable
;				  |+-------- OCIE1A : Timer/counter1 Output Compare A Match Interrupt Enable
;				  +--------- TOIE1 : Timer/Counter1 Overflow Interrupt Enable
;-----------------------------------------------------------------------------
.equ	Stand_TCCR0	=	0b00000100
;	Timer/Counter0 Control	  ||||||||
;				  |||||||+-- CS00 : Clock Select
;				  ||||||+--- CS01 : Clock Select
;				  |||||+---- CS02 : Clock Select
;				  ||||+----- WGM01 : Waveform Generation Mode
;				  |||+------ COM00 : Compare Match Output Mode
;				  ||+------- COM01 : Compare Match Output Mode
;				  |+-------- WGM00 : Waveform Generation Mode
;				  +--------- FOC0 : Force Output Compare
;
;		CS02	CS01	CS00
;		0	0	0	timer/counter0 is stopped
;		0	0	1	clk
;		0	1	0	clk/8
;		0	1	1	clk/64
;		1	0	0	clk/256
;		1	0	1	clk/1024
;		1	1	0	external pin T0; falling edge
;		1	1	1	external pin T0; rising edge
;-----------------------------------------------------------------------------
.def	acc		= r16
.def	acc2		= r17
.def	param		= r18
;-----------------------------------------------------------------------------
.equ	KeybIdle	= 0x00
.equ	KeybWaitOnPress	= 0x01
.equ	KeybReady	= 0x02
.equ	KeybActive	= 0x03
.equ	KeybWaitOnFree	= 0x04
.equ	KeybDelayLimit	= 0x18
;-----------------------------------------------------------------------------
.equ	KeyPort		= PORTD
.equ	LEDPort		= PORTB
.equ	KeyPDirection	= KeyPort - 1
.equ	LedPDirection	= LEDPort - 1
.equ	KeyPInput	= KeyPort - 2
;-----------------------------------------------------------------------------
.equ	Key0Pin		= 0
.equ	Key1Pin		= 1
.equ	Key2Pin		= 2
.equ	Key3Pin		= 3
.equ	Key4Pin		= 4
.equ	Key5Pin		= 5
.equ	Key6Pin		= 6
.equ	Key7Pin		= 7
;-----------------------------------------------------------------------------
.equ	Key0Code	= 'A'
.equ	Key1Code	= 'B'
.equ	Key2Code	= 'C'
.equ	Key3Code	= 'D'
.equ	Key4Code	= 'E'
.equ	Key5Code	= 'F'
.equ	Key6Code	= 'G'
.equ	Key7Code	= 'H'
;-----------------------------------------------------------------------------
	.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;						;
;						; VAR
KeybStatus :		.byte	1		;  KeybStatus : BYTE ;
KeybCode :		.byte	1		;  KeybCode : BYTE ;
KeybDelayCt :		.byte	1		;  KeybDelayCt : BYTE ;
OutputData :		.byte	1		;  OutputData : BYTE ;
;-----------------------------------------------------------------------------
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
	.org	0				;
	rjmp	ResetProcessor			;
;-----------------------------------------------------------------------------
	.org	INT0addr			;External Interrupt0 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT1addr			;External Interrupt1 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ICP1addr			;Input Capture1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Aaddr			;Output Compare1A Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC1Baddr			;Output Compare1B Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF1addr			;Overflow1 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OVF0addr			;Overflow0 Interrupt Vector Address
	rjmp	TC0OvfInterrupt			;
;-----------------------------------------------------------------------------
	.org	SPIaddr				;SPI Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	URXCaddr			;USART Receive Complete Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UDREaddr			;USART Data Register Empty Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	UTXCaddr			;USART Transmit Complete Int. Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ACIaddr				;Analog Comparator Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	INT2addr			;External Interrupt2 Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	OC0addr				;Output Compare0 Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	ERDYaddr			;EEPROM Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
	.org	SPMaddr				;SPM complete Interrupt Vector Address
	reti					;
;-----------------------------------------------------------------------------
.include "util.inc"
;-----------------------------------------------------------------------------
HardwareInit :					;PROCEDURE HardwareInit ;
;*************					;BEGIN (* HardwareInit *)
	ldi	acc,Stand_TIMSK			; [TIMSK] := Stand_TIMSK ;
	out	TIMSK,acc			;
	ldi	acc,Stand_TCCR0			; [TCCR0] := Stand_TCCR0 ;
	out	TCCR0,acc			;
	ldi	acc,0xFF			; [DDRA] := <output> ;
	out	DDRA,acc			;
	out	DDRB,acc			; [DDRB] := <output> ;
	out	DDRC,acc			; [DDRC] := <output> ;
	out	DDRD,acc			; [DDRD] := <output> ;
	out	PORTA,acc			; [PORTA] := 0xFF ;
	out	PORTB,acc			; [PORTB] := 0xFF ;
	out	PORTC,acc			; [PORTC] := 0xFF ;
	out	PORTD,acc			; [PORTD] := 0xFF ;
	ret					;END (* HardwareInit *) ;
;-----------------------------------------------------------------------------
EnvirInit :					;PROCEDURE EnvirInit ;
;**********					;BEGIN (* EnvirInit *)
	ldi	acc,0				; KeyPort := <input> + <pullup>;
	out	KeyPDirection,acc		;
	ldi	acc,0xFF			;
	out	KeyPort,acc			;
	out	LedPDirection,acc		; LEDPort := <output> + <outp.=high> ;
	out	LEDPort,acc			;
	ret					;END (* EnvirInit *) ;
;-----------------------------------------------------------------------------
SoftwareInit :					;PROCEDURE SoftwareInit ;
;*************					;BEGIN (* SoftwareInit *)
	ldi	acc,0				; KeybDelayCt := 0 ;
	sts	KeybDelayCt,acc			;
	ldi	acc,0xFF			; OutputData := 0xFF ;
	sts	OutputData,acc			;
	ldi	acc,KeybIdle			; KeybStatus := KeybIdle ;
	sts	KeybStatus,acc			;
	ret					;END (* SoftwareInit *) ;
;-----------------------------------------------------------------------------
TC0OvfInterrupt :				;PROCEDURE TC0OvfInterrupt ;
;****************				; VAR
;						;  Temp [acc2] : BYTE ;
;						;  Kbd [acc] : BYTE ;
;						;BEGIN (* TC0OvfInterrupt *)
	pushf					;
	pushz					;
	push	acc2				;
	ldz	KeybStatus			; CASE KeybStatus OF
	ld	acc,z				;
	cpi	acc,KeybIdle			; KeybIdle :
	brne	TC0OV_1				;  BEGIN
	in	acc,KeyPInput			;   IF KeyPInput <> 0xFF THEN
	cpi	acc,0xFF			;
	breq	TC0OV_11			;   BEGIN
	ldi	acc,KeybWaitOnPress		;    KeybStatus := KeybWaitOnPress ;
	st	z,acc				;
	ldi	acc,KeybDelayLimit		;    KeybDelayCt := KeybDelayLimit ;
	sts	KeybDelayCt,acc			;
TC0OV_11 :					;   END (* IF *) ;
	rjmp	TC0OV_0				;  END (* CASE VARIANT *)
TC0OV_1	:					;
	cpi	acc,KeybWaitOnPress		; KeybWaitOnPress :
	brne	TC0OV_2				;  BEGIN
	ldz	KeybDelayCt			;   KeybDelayCt := KeybDelayCt - 1 ;
	ld	acc,z				;
	dec	acc				;
	st	z,acc				;
	cpi	acc,0				;   IF KeybDelayCt = 0 THEN
	brne	TC0OV_21			;   BEGIN
	in	acc,KeyPInput			;    IF KeyPInput <> 0xFF THEN
	cpi	acc,0xFF			;
	breq	TC0OV_22			;    BEGIN
	ldi	acc,KeybReady			;     KeybStatus := KeybReady ;
	sts	KeybStatus,acc			;
	rjmp	TC0OV_23			;    END (* IF ... *)
TC0OV_22 :					;    ELSE
						;    BEGIN
	ldi	acc,KeybIdle			;     KeybStatus := KeybIdle ;
	sts	KeybStatus,acc			;
TC0OV_23 :					;    END (* IF ... ELSE *) ;
TC0OV_21 :					;   END (* IF *) ;
	rjmp	TC0OV_0				;  END (* CASE VARIANT *)
TC0OV_2	:					;
	cpi	acc,KeybReady			; KeybReady :
	brne	TC0OV_3				;  BEGIN
	ldi	acc2,0				;   Temp := 0 ;
	in	acc,KeyPInput			;   Kbd := KeyPInput ;
	sbrs	acc,Key0Pin			;   IF Kbd[Key0Pin] THEN
	ldi	acc2,Key0Code			;    Temp := Key0Code ;
	sbrs	acc,Key1Pin			;   IF Kbd[Key1Pin] THEN
	ldi	acc2,Key1Code			;    Temp := Key1Code ;
	sbrs	acc,Key2Pin			;   IF Kbd[Key2Pin] THEN
	ldi	acc2,Key2Code			;    Temp := Key2Code ;
	sbrs	acc,Key3Pin			;   IF Kbd[Key3Pin] THEN
	ldi	acc2,Key3Code			;    Temp := Key3Code ;
	sbrs	acc,Key4Pin			;   IF Kbd[Key4Pin] THEN
	ldi	acc2,Key4Code			;    Temp := Key4Code ;
	sbrs	acc,Key5Pin			;   IF Kbd[Key5Pin] THEN
	ldi	acc2,Key5Code			;    Temp := Key5Code ;
	sbrs	acc,Key6Pin			;   IF Kbd[Key6Pin] THEN
	ldi	acc2,Key6Code			;    Temp := Key6Code ;
	sbrs	acc,Key7Pin			;   IF Kbd[Key7Pin] THEN
	ldi	acc2,Key7Code			;    Temp := Key7Code ;
	sts	KeybCode,acc2			;   KeybCode := Temp ;
	ldi	acc,KeybActive			;   KeybStatus := KeybActive ;
	st	z,acc				;
	rjmp	TC0OV_0				;  END (* CASE VARIANT *)
TC0OV_3	:					;
	cpi	acc,KeybActive			; KeybActive :
	brne	TC0OV_4				;  BEGIN
	in	acc,KeyPInput			;   IF KeyPInput = 0xFF THEN
	cpi	acc,0xFF			;
	brne	TC0OV_41			;   BEGIN
	ldi	acc,KeybWaitOnFree		;    KeybStatus := KeybWaitOnFree ;
	st	z,acc				;
	ldi	acc,KeybDelayLimit		;    KeybDelayCt := KeybDelayLimit ;
	sts	KeybDelayCt,acc			;
TC0OV_41 :					;   END (* IF *) ;
	rjmp	TC0OV_0				;  END (* CASE VARIANT *)
TC0OV_4	:					;
	cpi	acc,KeybWaitOnFree		; KeybWaitOnFree :
	brne	TC0OV_5				;  BEGIN
	ldz	KeybDelayCt			;   KeybDelayCt := KeybDelayCt - 1 ;
	ld	acc,z				;
	dec	acc				;
	st	z,acc				;
	cpi	acc,0				;   IF KeybDelayCt = 0 THEN
	brne	TC0OV_51			;   BEGIN
	ldi	acc,KeybIdle			;    KeybStatus := KeybIdle ;
	sts	KeybStatus,acc			;
TC0OV_51 :					;   END (* IF *) ;
	rjmp	TC0OV_0				;  END (* CASE VARIANT *)
TC0OV_5	:					; ELSE
						;  BEGIN
						;  END (* CASE ELSE VARIANT *)
TC0OV_0	:					; END (* CASE *) ;
	pop	acc2				;
	popz					;
	popf					;
	reti					;END (* TC0OvfInterrupt *) ;
;-----------------------------------------------------------------------------
KeyPressed :					;FUNCTION KeyPressed ( ) : BOOLEAN [C] ;
;***********					;BEGIN
	lds	acc,KeybCode			; RETURN ( KeybCode <> 0 ) ;
	cpi	acc,0				;
	breq	KeyPr_0				;
	sec					;
	ret					;
KeyPr_0	:					;
	clc					;
	ret					;END (* KeyPressed *) ;
;-----------------------------------------------------------------------------
ReadKey :					;FUNCTION ReadKey ( ) : CHAR [acc] ;
;********					; VAR
;						;  Temp [acc] : CHAR ;
						;BEGIN
	ldz	KeybCode			; Temp := KeybCode ;
	ld	acc,z				;
	ldi	acc2,0				; KeybCode := 0 ;
	st	z,acc2				;
						; ReadKey := Temp ;
	ret					;END (* ReadKey *) ;
;-----------------------------------------------------------------------------
RunCommand :					;PROCEDURE RunCommand ( Ch [param] : CHAR ) ;
;***********					;BEGIN
	ldz	OutputData			;
	cpi	param,Key0Code			; IF Ch = Key0Code THEN
	brne	RuCmm_0				; BEGIN
	ld	acc,z				;  OutputData := OutputData EXOR 0b00000001 ;
	ldi	acc2,0b00000001			;
	eor	acc,acc2			;
	st	z,acc				;
	out	LEDPort,acc			;  LEDPort := OutputData ;
	ret					;  RETURN ;
RuCmm_0	:					; END (* IF *) ;
	cpi	param,Key1Code			; IF Ch = Key1Code THEN
	brne	RuCmm_1				; BEGIN
	ld	acc,z				;  OutputData := OutputData EXOR 0b00000010 ;
	ldi	acc2,0b00000010			;
	eor	acc,acc2			;
	st	z,acc				;
	out	LEDPort,acc			;  LEDPort := OutputData ;
	ret					;  RETURN ;
RuCmm_1	:					; END (* IF *) ;
	cpi	param,Key2Code			; IF Ch = Key2Code THEN
	brne	RuCmm_2				; BEGIN
	ld	acc,z				;  OutputData := OutputData EXOR 0b00000100 ;
	ldi	acc2,0b00000100			;
	eor	acc,acc2			;
	st	z,acc				;
	out	LEDPort,acc			;  LEDPort := OutputData ;
	ret					;  RETURN ;
RuCmm_2	:					; END (* IF *) ;
	cpi	param,Key3Code			; IF Ch = Key3Code THEN
	brne	RuCmm_3				; BEGIN
	ld	acc,z				;  OutputData := OutputData EXOR 0b00001000 ;
	ldi	acc2,0b00001000			;
	eor	acc,acc2			;
	st	z,acc				;
	out	LEDPort,acc			;  LEDPort := OutputData ;
	ret					;  RETURN ;
RuCmm_3	:					; END (* IF *) ;
	cpi	param,Key4Code			; IF Ch = Key4Code THEN
	brne	RuCmm_4				; BEGIN
	ld	acc,z				;  OutputData := OutputData EXOR 0b00010000 ;
	ldi	acc2,0b00010000			;
	eor	acc,acc2			;
	st	z,acc				;
	out	LEDPort,acc			;  LEDPort := OutputData ;
	ret					;  RETURN ;
RuCmm_4	:					; END (* IF *) ;
	cpi	param,Key5Code			; IF Ch = Key5Code THEN
	brne	RuCmm_5				; BEGIN
	ld	acc,z				;  OutputData := OutputData EXOR 0b00100000 ;
	ldi	acc2,0b00100000			;
	eor	acc,acc2			;
	st	z,acc				;
	out	LEDPort,acc			;  LEDPort := OutputData ;
	ret					;  RETURN ;
RuCmm_5	:					; END (* IF *) ;
	cpi	param,Key6Code			; IF Ch = Key6Code THEN
	brne	RuCmm_6				; BEGIN
	ld	acc,z				;  OutputData := OutputData EXOR 0b01000000 ;
	ldi	acc2,0b01000000			;
	eor	acc,acc2			;
	st	z,acc				;
	out	LEDPort,acc			;  LEDPort := OutputData ;
	ret					;  RETURN ;
RuCmm_6	:					; END (* IF *) ;
	cpi	param,Key7Code			; IF Ch = Key7Code THEN
	brne	RuCmm_7				; BEGIN
	ld	acc,z				;  OutputData := OutputData EXOR 0b10000000 ;
	ldi	acc2,0b10000000			;
	eor	acc,acc2			;
	st	z,acc				;
	out	LEDPort,acc			;  LEDPort := OutputData ;
	ret					;  RETURN ;
RuCmm_7	:					; END (* IF *) ;
	ret					;END (* RunCommand *) ;
;-----------------------------------------------------------------------------
ResetProcessor :				;
	ldi	acc,HIGH(RAMEND)		;
	out	SPH,acc				;
	ldi	acc,LOW(RAMEND)			;
	out	SPL,acc				;
						;BEGIN (* Main *)
	rcall	ClearRAM			; ClearRAM ( ) ;
	rcall	ClearRegs			; ClearRegs ( ) ;
	rcall	HardwareInit			; HardwareInit ( ) ;
	rcall	EnvirInit			; EnvirInit ( ) ;
	rcall	SoftwareInit			; SoftwareInit ( ) ;
	sei					; EI ( ) ;
Main_0	:					; LOOP
	rcall	KeyPressed			;  IF KeyPressed ( ) THEN
	brcc	Main_1				;  BEGIN
	rcall	ReadKey				;   RunCommand ( ReadKey ( ) ) ;
	mov	param,acc			;
	rcall	RunCommand			;
Main_1	:					;  END (* IF *) ;
	rjmp	Main_0				; END (* LOOP *) ;
						;END (* Main *) ;
;-----------------------------------------------------------------------------
.exit
