;
;*******************************************************
;*******************************************************
;**                                                   **
;**    Sztuka programowania mikrokontrolerow AVR      **
;**                                                   **
;** Biblioteka procedur obslugi USART w przerwaniach  **
;**                                                   **
;**       Andrzej Pawluczuk                           **
;**                                                   **
;**         Mikrokontroler: ATMEGA8515                **
;**                Bialystok, 2006                    **
;**                                                   **
;*******************************************************
;*******************************************************
;
.message	"***>>> Dolaczenie bibioteki obslugi USART"
;
;*******************************************************
;*******************************************************
;
;  UWAGA:
;  Przed wlaczeniem biblioteki procedur do programu
;  nalezy w programie zdefiniowac nastepujace stale
;  TXDBuffSize - okreslenie wielkosci bufora cyklicznego nadajnika
;  RXDBuffSize - okreslenie wielkosci bufora cyklicznego odbiornika
;
;  USART_LED_Trace=0 jezeli nie sa obslugiwane LEDY sygnalizujace transmisje
;  USART_LED_Trace=1 jezeli sa obslugiwane LEDY sygnalizujace transmisje
;                   Brak definicji jest rownoznaczny z USART_LED_Trace=0
;
;  RXDLED - numer pinu w porcie do sygnalizacji odbierania
;  TXDLED - numer pinu w porcie do sygnalizacji nadawania
;  USARTTracePort - port, do ktorego przylaczone sa LED'y
;  LEDOnState == 0 - oznacza, ze LED jest wlaczany do swiecenia stanem LOW
;  LEDOnState == 1 - oznacza, ze LED jest wlaczany do swiecenia stanem HIGH
;
;  oraz zdefiniowac nastepujace symbole (jako rejestry z grupy R16..R31)
;     acc
;     acc2
;     param
;
;*******************************************************
;   Przykladowo:
; .def acc    = r16
; .def acc2   = r17
; .def param  = r18
; .equ TXDBuffSize = 16
; .equ RXDBuffSize = 8
; .equ RXDLED = 7
; .equ TXDLED = 6
; .equ LEDOnState = 0
;
;*******************************************************
;*******************************************************
;
.ifndef TXDBuffSize
	.error	"*****> Blad: brak definicji stalej TXDBuffSize"
	.exit
.endif
.ifndef RXDBuffSize
	.error	"*****> Blad: brak definicji stalej RXDBuffSize"
	.exit
.endif
;
;-----------------------------------------------------------------------------
.if USART_LED_Trace == 1
  .message	"***>>> Obsluga USART z obsluga sygnalizacji transmisji"
.else
  .message	"***>>> Obsluga USART bez obslugi sygnalizacji transmisji"
.endif
;
;-----------------------------------------------------------------------------
.equ	LibraryStand_UCR	=	0b11011000
;	USART Control Register		  ||||||||
;					  |||||||+-- TXD8 : Transmit Data Bit 8
;					  ||||||+--- RXB8 : Receive Data Bit 8
;					  |||||+---- CHR9 : 9-bit Characters
;					  ||||+----- TXEN : Transmitter Enable
;					  |||+------ RXEN : Receiver Enable
;					  ||+------- UDRIE : USART Data Register Empty Interrupt Enable
;					  |+-------- TXCIE : TX Complete Interrupt Enable
;					  +--------- RXCIE : RX Complete Interrupt Enable
;-----------------------------------------------------------------------------
.if USART_LED_Trace == 1
	.ifndef RXDLED
		.error	"*****> Blad: brak definicji stalej RXDLED"
		.exit
	.endif
	.ifndef TXDLED
		.error	"*****> Blad: brak definicji stalej TXDLED"
		.exit
	.endif
	.ifndef USARTTracePort
		.error	"*****> Blad: brak definicji stalej USARTTracePort"
		.exit
	.endif
	.ifndef LEDOnState
		.error	"*****> Blad: brak definicji stalej LEDOnState"
		.exit
	.endif
.endif
		.dseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  DATA  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
;					; CONST
.equ	IdleState	=	0	;  IdleState = 0 ;
.equ	BusyState	=	1	;  BusyState = 1 ;
.equ	StandLEDDelay	=	40	;  StandLEDDelay = 40 ;
;					;
;					; TYPE
.if USART_LED_Trace == 1
;					;  TXDRecT = RECORD
.equ	TXDRec_CycFr	=	0	;             CycFr   : BYTE ;
.equ	TXDRec_CycTo	=	1	;             CycTo   : BYTE ;
.equ	TXDRec_State	=	2	;             State   : BYTE ;
.equ	TXDRec_Delay	=	3	;             Delay   : BYTE ;
.equ	TXDRec_Buffer	=	4	;             Buffer  : ARRAY [ 0 .. TXDBuffSize ] OF CHAR ;
;					;            END (* RECORD *) ;
;					;  RXDRecT = RECORD
.equ	RXDRec_CycFr	=	0	;             CycFr   : BYTE ;
.equ	RXDRec_CycTo	=	1	;             CycTo   : BYTE ;
.equ	RXDRec_Delay	=	2	;             Delay   : BYTE ;
.equ	RXDRec_Buffer	=	3	;             Buffer  : ARRAY [ 0 .. RXDBuffSize ] OF CHAR ;
;					;            END (* RECORD *) ;
.equ	TXDRecSize =	TXDBuffSize+4	;
.equ	RXDRecSize =	RXDBuffSize+3	;
;					;
.else
;					;  TXDRecT = RECORD
.equ	TXDRec_CycFr	=	0	;             CycFr   : BYTE ;
.equ	TXDRec_CycTo	=	1	;             CycTo   : BYTE ;
.equ	TXDRec_State	=	2	;             State   : BYTE ;
.equ	TXDRec_Buffer	=	3	;             Buffer  : ARRAY [ 0 .. TXDBuffSize ] OF CHAR ;
;					;            END (* RECORD *) ;
;					;  RXDRecT = RECORD
.equ	RXDRec_CycFr	=	0	;             CycFr   : BYTE ;
.equ	RXDRec_CycTo	=	1	;             CycTo   : BYTE ;
.equ	RXDRec_Buffer	=	2	;             Buffer  : ARRAY [ 0 .. RXDBuffSize ] OF CHAR ;
;					;            END (* RECORD *) ;
.equ	TXDRecSize =	TXDBuffSize+3	;
.equ	RXDRecSize =	RXDBuffSize+2	;
.endif
;					; VAR
TXDRec:		.byte	TXDRecSize	;  TXDRec : TXDRecT ;
RXDRec:		.byte	RXDRecSize	;  RXDRec : RXDRecT ;
;					;
;					;
;-----------------------------------------------------------------------------
.if USART_LED_Trace == 1
.equ	USARTTracePDir	=	USARTTracePort - 1
.endif
	.cseg
;*******************************************************
;*******************************************************
;**                                                   **
;**                  CODE  SEGMENT                    **
;**                                                   **
;*******************************************************
;*******************************************************
USARTHardwareInit :				;PROCEDURE USARTHardwareInit ( Speed [param] : BYTE ) ;
;*****************				;BEGIN
	ldi	acc,LibraryStand_UCR		; [UCR] := LibraryStand_UCR ;
	out	UCR,acc				;
	out	UBRR,param			; [UBRR] := Speed ;
	in	acc,UDR				;
	cbi	DDRD,PD0			; [PORTD.PD0/RXD/] := [input] ;
	sbi	DDRD,PD1			; [PORTD.PD1/TXD/] := [output] ;
.if USART_LED_Trace == 1
	sbi	USARTTracePDir,RXDLED		; [USARTTracePort.RXDLED] := [output] ;
	sbi	USARTTracePDir,TXDLED		; [USARTTracePort.TXDLED] := [output] ;
  .if LEDOnState == 0
	sbi	USARTTracePort,RXDLED		; [USARTTracePort.RXDLED] := 1 ; /* LED Off */
	sbi	USARTTracePort,TXDLED		; [USARTTracePort.TXDLED] := 1 ; /* LED Off */
  .else
	cbi	USARTTracePort,RXDLED		; [USARTTracePort.RXDLED] := 0 ; /* LED Off */
	cbi	USARTTracePort,TXDLED		; [USARTTracePort.TXDLED] := 0 ; /* LED Off */
  .endif		
.endif
	rcall	USARTSoftwareInit		; USARTSoftwareInit ( ) ;
	ret					;END (* USARTHardwareInit *) ;
;-----------------------------------------------------------------------------
USARTSoftwareInit :				;PROCEDURE USARTSoftwareInit ;
;*****************				;BEGIN
	clr	acc				; RXDRec . CycFr := 0 ;
	ldz	RXDRec				;
	std	z+RXDRec_CycFr,acc		;
	std	z+RXDRec_CycTo,acc		; RXDRec . CycTo := 0 ;
.if USART_LED_Trace == 1
	std	z+RXDRec_Delay,acc		; RXDRec . Delay := 0 ;
.endif
	ldz	TXDRec				; TXDRec . CycFr := 0 ;
	std	z+TXDRec_CycFr,acc		;
	std	z+TXDRec_CycTo,acc		; TXDRec . CycTo := 0 ;
.if USART_LED_Trace == 1
	std	z+TXDRec_Delay,acc		; TXDRec . Delay := 0 ;
.endif
	ldi	acc,IdleState			; TXDRec . State := IdleState ;
	std	z+TXDRec_State,acc		;
	ret					;END (* USARTSoftwareInit *) ;
;-----------------------------------------------------------------------------
.if USART_LED_Trace == 1
USARTTimeService	:			;PROCEDURE USARTTimeService  ;
;****************				;BEGIN
	pushz					;
	ldz	RXDRec+RXDRec_Delay		; IF RXDRec . Delay <> 0 THEN
	ld	acc,z				;
	cpi	acc,0				;
	breq	UTimSe_0			;  BEGIN
	dec	acc				;   RXDRec . Delay := RXDRec . Delay - 1 ;
	st	z,acc				;
	cpi	acc,0				;   IF RXDRec . Delay = 0 THEN
	brne	UTimSe_1			;    BEGIN
  .if LEDOnState == 0
	sbi	USARTTracePort,RXDLED		;     [USARTTracePort] [ RXDLED ] := 1 ;
  .else
	cbi	USARTTracePort,RXDLED		;     [USARTTracePort] [ RXDLED ] := 0 ;
  .endif
UTimSe_1 :					;    END (* IF *) ;
UTimSe_0 :					;  END (* IF *) ;
	ldz	TXDRec+TXDRec_Delay		; IF TXDRec . Delay <> 0 THEN
	ld	acc,z				;
	cpi	acc,0				;
	breq	UTimSe_2			;  BEGIN
	dec	acc				;   TXDRec . Delay := TXDRec . Delay - 1 ;
	st	z,acc				;
	cpi	acc,0				;   IF TXDRec . Delay = 0 THEN
	brne	UTimSe_3			;    BEGIN
  .if LEDOnState == 0
	sbi	USARTTracePort,TXDLED		;     [USARTTracePort] [ TXDLED ] := 1 ;
  .else
	cbi	USARTTracePort,TXDLED		;     [USARTTracePort] [ TXDLED ] := 0 ;
  .endif
UTimSe_3 :					;    END (* IF *) ;
UTimSe_2 :					;  END (* IF *) ;
	popz					;
	ret					;END (* USARTTimeService *) ;
.endif
;-----------------------------------------------------------------------------
USARTRXDInterrupt :				;PROCEDURE USARTRXDInterrupt ;
;*****************				;BEGIN
	pushf					;
	pushz					;
	ldz	RXDRec				;
.if USART_LED_Trace == 1
  .if LEDOnState == 0
	cbi	USARTTracePort,RXDLED		; [USARTTracePort] [ RXDLED ] := 0 ;
  .else
	sbi	USARTTracePort,RXDLED		; [USARTTracePort] [ RXDLED ] := 1 ;
  .endif
	ldi	acc,StandLEDDelay		; RXDRec . Delay := StandLEDDelay ;
	std	z+RXDRec_Delay,acc		;
.endif
	ldd	acc,z+RXDRec_CycTo		; RXDRec . Buffer [ RXDRec . CycTo ] := [UDR] ;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	in	acc,UDR				;
	std	z+RXDRec_Buffer,acc		;
	ldz	RXDRec+RXDRec_CycTo		; RXDRec . CycTo := RXDRec . CycTo + 1 ;
	ld	acc,z				;
	inc	acc				;
	cpi	acc,RXDBuffSize			; IF RXDRec . CycTo >= RXDBuffSize THEN
	brcs	URXDInt_0			;  BEGIN
	ldi	acc,0				;   RXDRec . CycTo := 0 ;
URXDInt_0 :					;  END (* IF *) ;
	st	z,acc				;
	popz					;
	popf					;
	reti					;END (* USARTRXDInterrupt *) ;
;-----------------------------------------------------------------------------
USARTDataRegEmptInterrupt :			;PROCEDURE USARTDataRegEmptInterrupt ;
;*************************			;BEGIN
	reti					;END (* USARTDataRegEmptInterrupt *) ;
;-----------------------------------------------------------------------------
USARTTXDInterrupt :				;PROCEDURE USARTTXDInterrupt ;
;*****************				;BEGIN
	pushf					;
	pushz					;
	push	acc2				;
	ldz	TXDRec				; IF TXDRec . CycFr = TXDRec . CycTo THEN
	ldd	acc,z+TXDRec_CycFr		;
	ldd	acc2,z+TXDRec_CycTo		;
	cp	acc,acc2			;
	brne	UTXD0I_0			;  BEGIN
	ldi	acc,IdleState			;   TXDRec . State := IdleState ;
	std	z+TXDRec_State,acc		;
	rjmp	UTXD0I_1			;  END (* IF ... *)
UTXD0I_0 :					; ELSE
						;  BEGIN
.if USART_LED_Trace == 1
	ldi	acc2,StandLEDDelay		;   TXDRec . Delay := StandLEDDelay ;
	std	z+TXDRec_Delay,acc2		;
.endif
	add	zl,acc				;   [UDR] = TXDRec . Buffer [ TXDRec . CycFr ] ;
	ldi	acc,0				;
	adc	zh,acc				;
	ldd	acc,z+TXDRec_Buffer		;
	out	UDR,acc				;
	ldz	TXDRec+TXDRec_CycFr		;   TXDRec . CycFr := TXDRec . CycFr + 1 ;
	ld	acc,z				;
	inc	acc				;
	cpi	acc,TXDBuffSize			;   IF TXDRec . CycFr >= TXDBuffSize THEN
	brcs	UTXD0I_2			;    BEGIN
	ldi	acc,0				;     TXDRec . CycFr := 0 ;
UTXD0I_2 :					;    END (* IF *) ;
	st	z,acc				;
UTXD0I_1 :					;  END (* IF ... ELSE *) ;
	pop	acc2				;
	popz					;
	popf					;
	reti					;END (* USARTTXDInterrupt *) ;
;-----------------------------------------------------------------------------
SendSerial :					;PROCEDURE SendSerial ( SerialCh [param] : CHAR ) ;
;***********					; VAR
;						;  NextInx[acc] : CHAR ;
;						;BEGIN
	pushz					;
	cbi	UCR,TXCIE			; DI ( Serial ) ;
	ldz	TXDRec				;
	ldd	acc,z+TXDRec_State		; IF TXDRec . State = IdleState THEN
	cpi	acc,IdleState			;
	brne	SenSer_0			;  BEGIN
	ldi	acc,BusyState			;   TXDRec . State := BusyState ;
	std	z+TXDRec_State,acc		;
.if USART_LED_Trace == 1
  .if LEDOnState == 0
	cbi	USARTTracePort,TXDLED		;   [USARTTracePort] [ TXDLED ] := 0 ;
  .else
	sbi	USARTTracePort,TXDLED		;   [USARTTracePort] [ TXDLED ] := 1 ;
  .endif
	ldi	acc,StandLEDDelay		;   TXDRec . Delay := StandLEDDelay ;
	std	z+TXDRec_Delay,acc		;
.endif
	out	UDR,param			;   [UDR] = SerialCh ;
	sbi	UCR,TXCIE			;   EI ( Serial ) ;
	popz					;   RETURN ;
	ret					;
SenSer_0 :					;  END (* IF *) ;
SenSer_1 :					; LOOP
	ldd	acc,z+TXDRec_CycTo		;  NextInx := TXDRec . CycTo + 1 ;
	inc	acc				;
	cpi	acc,TXDBuffSize			;  IF NextInx >= TXDBuffSize THEN
	brcs	SenSer_4			;   BEGIN
	ldi	acc,0				;    NextInx := 0 ;
SenSer_4 :					;   END (* IF *) ;
	ldd	acc2,z+TXDRec_CycFr		;  IF NextInx <> TXDRec . CycFr THEN
	cp	acc,acc2			;
						;   BEGIN
	brne	SenSer_3			;    EXIT ;
						;   END (* IF *) ;
	sbi	UCR,TXCIE			;  EI ( Serial ) ;
	nop					;  ;
	nop					;  ;
	cbi	UCR,TXCIE			;  DI ( Serial ) ;
	rjmp	SenSer_1			; END (* LOOP *) ;
SenSer_3 :					;
	ldd	acc,z+TXDRec_CycTo		; TXDRec . Buffer [ TXDRec . CycTo ] := SerialCh ;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	std	z+TXDRec_Buffer,param		;
	ldz	TXDRec+TXDRec_CycTo		; TXDRec . CycTo := TXDRec . CycTo + 1 ;
	ld	acc,z				;
	inc	acc				;
	cpi	acc,TXDBuffSize			; IF TXDRec . CycTo >= TXDBuffSize THEN
	brcs	SenSer_5			;  BEGIN
	ldi	acc,0				;   TXDRec . CycTo := 0 ;
SenSer_5 :					;  END (* IF *) ;
	st	z,acc				;
	sbi	UCR,TXCIE			; EI ( Serial ) ;
	popz					;
	ret					;END (* SendSerial *) ;
;-----------------------------------------------------------------------------
SendSerialFlashTxt :				;PROCEDURE SendSerialFlashTxt ( Text [z] : POINTER TO CHAR ) ;
;*******************				; VAR
;						;  Ch[r0] : CHAR ;
;						;BEGIN
SenSFTxt_0 :					; LOOP
	lpm					;  Ch := Text ^ ;
	adiw	zl,1				;  Text := Text + 1 ;
	mov	param,r0			;  IF Ch = CHR ( 0 ) THEN
	cpi	param,0				;
	brne	SenSFTxt_1			;   BEGIN
	ret					;    RETURN ;
SenSFTxt_1 :					;   END (* IF *) ;
	rcall	SendSerial			;  SendSerial ( Ch ) ;
	rjmp	SenSFTxt_0			; END (* LOOP *) ;
						;END (* SendSerialFlashTxt *) ;
;-----------------------------------------------------------------------------
SendSerialRAMTxt :				;PROCEDURE SendSerialRAMTxt ( Text [z] : POINTER TO CHAR ) ;
;*****************				; VAR
;						;  Ch[param] : CHAR ;
;						;BEGIN
SenSRTxt_0 :					; LOOP
	ld	param,z+			;  Ch := Text ^ ;
						;  Text := Text + 1 ;
	cpi	param,0				;  IF Ch = CHR ( 0 ) THEN
	brne	SenSRTxt_1			;   BEGIN
	ret					;    RETURN ;
SenSRTxt_1 :					;   END (* IF *) ;
	rcall	SendSerial			;  SendSerial ( Ch ) ;
	rjmp	SenSRTxt_0			; END (* LOOP *) ;
						;END (* SendSerialRAMTxt *) ;
;-----------------------------------------------------------------------------
SerialDataPresent :				;FUNCTION SerialDataPresent ( ) : BOOLEAN ;
;******************				;BEGIN
	ldz	RXDRec				; IF RXDRec . CycFr = RXDRec . CycTo THEN
	ldd	acc,z+RXDRec_CycFr		;
	ldd	acc2,z+RXDRec_CycTo		;
	cp	acc,acc2			;
	brne	SeDaPre_0			;  BEGIN
	clc					;   SerialDataPresent := FALSE ;
	ret					;   RETURN ;
SeDaPre_0 :					;  END (* IF *) ;
	sec					; SerialDataPresent := TRUE ;
	ret					;END (* SerialDataPresent *) ;
;-----------------------------------------------------------------------------
GetSerial :					;FUNCTION GetSerial ( ) : CHAR [acc] ;
;**********					; VAR
;						;  Ch[stack] : CHAR ;
;						;BEGIN
	ldz	RXDRec				; Ch := RXDRec . Buffer [ RXDRec . CycFr ] ;
	ldd	acc,z+RXDRec_CycFr		;
	add	zl,acc				;
	ldi	acc,0				;
	adc	zh,acc				;
	ldd	acc,z+RXDRec_Buffer		;
	push	acc				;
	ldz	RXDRec+RXDRec_CycFr		; RXDRec . CycFr := RXDRec . CycFr + 1 ;
	ld	acc,z				;
	inc	acc				;
	cpi	acc,RXDBuffSize			; IF RXDRec . CycFr >= RXDBuffSize THEN
	brcs	GetSer_0			;  BEGIN
	ldi	acc,0				;   RXDRec . CycFr := 0 ;
GetSer_0 :					;  END (* IF *) ;
	st	z,acc				;
	pop	acc				; GetSerial := Ch ;
	ret					;END (* GetSerial *) ;
;-----------------------------------------------------------------------------
.exit
